/*
 * This is a simple program that merges the input from the
 * first universe of the input plugins and outputs it to the
 * output plugin.
 */
#include <stdio.h>
#include <string.h>
#include <dmx/plugin.h>
#include <pthread.h>

static dmxplugin_t   *outputplugin = NULL;

static int            numplugins = 0;
static dmxplugin_t   *plugins[10];
static dmxuniverse_t  universes[10];

/*
 * this mutex saves the critical section in which the
 * universes are read and merged.
 */
static pthread_mutex_t mut = PTHREAD_MUTEX_INITIALIZER;


static int my_notify (dmxuniverse_t *u)
{
  if (u && outputplugin && outputplugin->send_universe)
    {
      int i;
      dmxuniverse_t target;

      memset (&target,0,sizeof(target));
      target.numslots = 512;

      pthread_mutex_lock(&mut);

      for (i=0; i<numplugins && i<10; i++)
        {
          int k;
          for (k=0; k<universes[i].numslots; k++)
            if (universes[i].buffer[k] > target.buffer[k])
              target.buffer[k] = universes[i].buffer[k];
        }

      if (outputplugin->send_universe (0, &target) < 0)
        printf ("send_universe failed\n");

      pthread_mutex_unlock(&mut);
    }
  return 0;
}



int main(int argc, char **argv)
{
  int i;

  if (argc<3)
    {
      printf ("%s <outputplugin> <plugin1> [ <plugin-N> ]+\n", argv[0]);
      exit(0);
    }

  outputplugin = loadPlugin (NULL, argv[1]);
  if (!outputplugin)
    {
      printf ("unable to load plugin '%s' for output\n", argv[1]);
      exit(0);
    }

  for (i=2; i<argc; i++)
    {
      plugins[numplugins] = loadPlugin (NULL, argv[i]);
      if (plugins[numplugins])
        {
          memset (&universes[numplugins], 0, sizeof(dmxuniverse_t));
          universes[numplugins].notify = my_notify;
          if (plugins[numplugins]->bind_universe(0, &universes[numplugins]) < 0)
            printf ("error binding universe to plugin %s\n", argv[i]);
          numplugins++;
        }
      else
        printf ("unable to load plugin %s\n", argv[i]);
    }

  if (numplugins > 0)
    {
      char buffer[10];
      fgets (buffer, sizeof(buffer), stdin);
    }

  for (i=0; i<numplugins; i++)
    {
      plugins[i]->unbind_universe(0, &universes[i]);
      plugins[i]->delete(plugins[i]);
    }

  if (outputplugin)
    outputplugin->delete(outputplugin);

  exit(0);
}

