#!/bin/sh
#
# This installs all files needed for usage and development.
#
#----------------------------------------------
# some config stuff

. .config

# PLUGIN_LOCATION is defined in .config. This enables having it defined only once.
PLUGINS="plugins/*/*_plugin.so"

LIB_LOCATION="/usr/local/lib"
LIBS="lib/libdmxplugin.a"

INCLUDE_LOCATION="/usr/local/include/dmx"
INCLUDES="include/dmx/plugin.hh  include/dmx/plugin.h"

BIN_LOCATION="/usr/local/bin"

#----------------------------------------------
if ls $PLUGINS >/dev/null 2>/dev/null
then
  if mkdir -p $PLUGIN_LOCATION
  then
    if install -m664 $PLUGINS $PLUGIN_LOCATION
    then
      echo "installed into $PLUGIN_LOCATION:"
      for i in $PLUGINS ; do echo "    $i" ; done
    else
      echo "ERROR: failed to install $PLUGINS to $PLUGIN_LOCATION"
    fi
  else
    echo 'ERROR: unable to create plugin directory:' $PLUGIN_LOCATION
    exit 1
  fi
else
  echo "ERROR: no plugins found to install (did you compile them)."
  exit 1
fi



if ls $LIBS >/dev/null 2>/dev/null
then
  if mkdir -p $LIB_LOCATION
  then
    if install -m644 $LIBS $LIB_LOCATION
    then
      echo "installed into $LIB_LOCATION:"
      for i in $LIBS ; do echo "    $LIBS" ; done
    else
      echo "ERROR: failed to install $LIBS to $LIB_LOCATION"
    fi
  else
    echo 'ERROR: unable to create library directory'
    exit 1
  fi
else
  echo 'ERROR: no libraries found to install'
  exit 1
fi


if ls $INCLUDES >/dev/null 2>/dev/null
then
  if mkdir -p $INCLUDE_LOCATION
  then
    if install -m644 $INCLUDES $INCLUDE_LOCATION
    then
      echo "installed $INCLUDE_LOCATION:"
      for i in $INCLUDES ; do echo "    $INCLUDES" ; done
    else
      echo "ERROR: failed to install $INCLUDES to $INCLUDE_LOCATION"
    fi
  else
    echo 'ERROR: unable to create library directory'
    exit 1
  fi
else
  echo 'ERROR: not all include files found to install'
  exit 1
fi


CONFIGFILE=/etc/dmxplugin.conf
OLDCONFIGFILE=/etc/dmxplugin.conf.old

if [ -f "$CONFIGFILE" ]
then
  echo "Config file exists, renameing $CONFIGFILE to $OLDCONFIGFILE"
  mv "$CONFIGFILE" "$OLDCONFIGFILE"
fi

echo "plugindir=$PLUGIN_LOCATION" >$CONFIGFILE

echo "Created configuration in file $CONFIGFILE";

echo ; echo "INSTALLED ALL FILES SUCCESSLULLY" ; echo

for i in dmxconsole dmxd dmxdump lsdmxplugins dmxpluginforward dmxplugininfo
do
  if [ -f ./$i ]
  then
    if install -m755 $i $BIN_LOCATION
    then
      echo "installed $i into $BIN_LOCATION"
    else
      echo "failed to install $i into $BIN_LOCATION"
    fi
  else
    echo "ERROR: $i does not exist (didn't you compile it?)."
  fi
done

# EOF
