/*
 *
 *
 *
 *  external constraints: DMXPLUGIN_LOCATION must be defined by make.
 */

#include <stdio.h>
#include <dlfcn.h>

#include <dmx/plugin.h>

#include <trycatch.h>


static int deletePlugin (dmxplugin_t *p)
{
  if (p)
    {
      if (p->finalize)
        p->finalize();

      dlclose(p->handle);
      free(p);
    }
}

dmxplugin_t *loadPlugin (char *path, char *name)
{
  dmxplugin_t *p = (dmxplugin_t *)malloc (sizeof(dmxplugin_t));
  if (p)
    {
      char filename[FILENAME_MAX];
      sprintf (filename, "%s%s_plugin.so", DMXPLUGIN_LOCATION, name);

#if 0
      printf ("load plugin: %s\n", filename);
#endif
      p->handle = dlopen (filename, RTLD_NOW);
      if (p->handle)
        {
#if 0
          printf ("plugin %s has handle %p\n", filename, p->handle);
#endif

          p->initialize      = dlsym(p->handle, "initialize");
          if (p->initialize)
            {
              TRY
                {
                  if (p->initialize () >= 0)
                    {
                      char *error;

                      p->finalize        = dlsym(p->handle, "finalize");
                      p->bind_universe   = dlsym(p->handle, "bind_universe");
                      p->unbind_universe = dlsym(p->handle, "unbind_universe");
                      p->send_universe   = dlsym(p->handle, "send_universe");
                      p->get_plugin_info = dlsym(p->handle, "get_plugin_info");

                      if ((error = dlerror()))
                        fprintf (stderr, "symbol lookup failed: %s\n", error);
                      else
                        {
                          p->createUniverse = NULL;
                          p->delete = deletePlugin;
                          catchreturn(p);
                        }
                    }
                  else
                    fprintf (stderr, "unable to initialize plugin\n");
                }
              CATCH
                {
                  fprintf (stderr, "plugin crashed - load failed\n");
                }
            }
          else
            fprintf (stderr, "unable to find symbol initialize in plugin\n");
          dlclose(p->handle);
        }
      else
        {
          char *error = dlerror();
          fprintf (stderr, "unable to open shared library: %s\n", error?error:"unknown error");
        }
      free(p);
    }
  else
    fprintf (stderr, "no space left for plugin object\n");
  return NULL;
}


