
#include "dmxoe_codec.h"


bool
ready(int sock) {
  struct timeval nowait = {0, 0};
  int status;
  fd_set set;
  FD_ZERO(&set);
  FD_SET(sock, &set);
  status = select(sock+1, &set, NULL, NULL, &nowait);
  if(status == -1) perror("select error"), exit(1);
  return status > 0;
}


int
recv_udp(U16 port_in, U8 pktbuf[], unsigned pktlen) {
  static int sock = -1;
  struct sockaddr_in local;
  int n = sizeof(local);
  if(sock == -1) {
    sock = socket(AF_INET, SOCK_DGRAM, 0);
    if(sock < 0) perror("socket error"), exit(1);
    bzero(&local, sizeof(local));
    local.sin_family = AF_INET;
    local.sin_port = htons(port_in);
    if(bind(sock, (struct sockaddr*)&local, sizeof(local)) <0)
      perror("bind error"), exit(1);
  }
  if(ready(sock))
    return recvfrom(sock, pktbuf, pktlen, 0, &local, &n);
  else
    return 0;
}


void
watch(U16 port_in) {
  U8 netbuf[0x5000];
  U8 pktbuf[0x800];
  int port_status;
  int pktlen = -1;
  time_t ticks;
  header* phead = (header*)pktbuf;
  unsigned key;
  int index = 0;
  bool flags[256];

  bzero(netbuf, sizeof(netbuf));
  memset(flags, TRUE, sizeof(flags));
  for(;;) {
    while((key = getch()) != ERR) {
      switch(key) {
        case 'q':
          return;
        case KEY_DOWN:
          index += 0x10;
          break;
        case KEY_UP:
          index -= 0x10;
          index = max(index, 0);
          break;
        default:
          if(key < 256)
            flags[key] = !flags[key];
      }
    }
    port_status = recv_udp(port_in, pktbuf, sizeof(pktbuf));
    if(port_status < 0) perror("error receiving"), exit(1);
    if(port_status > 0) {
      ticks = time((time_t*)NULL);
      pktlen = port_status;
      shownet_decode(pktbuf, pktlen, netbuf, sizeof(netbuf));
    }
    erase();
    move(0, 0);
    printw("ShowNet dump \nUDP/%d -- ", port_in);
    if(pktlen < 0) {
      printw("waiting for first packet...", port_in);
      continue;
    }
    printw("%d bytes at %s", pktlen, ctime(&ticks));
    if(flags['h']) {
      display_header(phead);
    }
    if(flags['e']) {
      printw("\n\nrle encoded dimmer values\n");
      hex_dump(pktbuf+sizeof(header), pktlen-sizeof(header));
    }
    if(flags['d']) {
      printw("\n");
      hex_dump(netbuf+index, 0x200);
    }
  }
}


int
main(int argc, char* argv[])
{
  int port_in = 2501;
  if(argc>1) {
    port_in = atoi(argv[1]);
  }
  initscr();
  noecho();
  nodelay(stdscr, TRUE);
  keypad(stdscr, TRUE);
  watch(port_in);
  endwin();
  return 0;
}

